""" surl.services """

#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.

import xml.dom.minidom
import re
try:
    import json
except ImportError:
    import simplejson as json

from surl import core

def supportedServices():
    """ Returns a dictionary with the supported surl services """
    dict_service = {
        'a.gd'         : a_gd(),
        'bit.ly'       : bit_ly(),
        'burnurl.com'  : burnurl_com(),
        'cli.gs'       : cli_gs(),
        'decenturl.com': decenturl_com(),
        'digg.com'     : digg_com(),
        'is.gd'        : is_gd(),
        'kl.am'        : kl_am(),
        'liip.to'      : liip_to(),
        'metamark.net' : metamark_net(),
        'sn.im'        : sn_im('sn.im'),
        'snipr.com'    : sn_im('snipr.com'),
        'snipurl.com'  : sn_im('snipurl.com'),
        'snurl.com'    : sn_im('snurl.com'),
        'tinyurl.com'  : tinyurl_com(),
        'tr.im'        : tr_im(),
        'turl.ca'      : turl_ca(),
        'ur.ly'        : ur_ly(),
        'zz.gd'        : zz_gd(),
    }
    return dict_service

class tinyurl_com:
    #http://tinyurl.com/api-create.php?url=http://example.com/
    #GET or POST
    #Notes: Soon custom alias (title)
    def get(self, args, url):
        values = {
            'url': url,
        }
        service_link = "http://tinyurl.com/api-create.php"
        result = core().getShortenedLink(service_link, url, values)
        return result

class a_gd:
    #curl "http://a.gd/?module=ShortURL&file=Add&mode=API" --data "url=http://example.com"
    #POST
    #TODO: Check for custom alias title
    def get(self, args, url):
        values = {
            'url': url,
        }
        service_link = "http://a.gd/?module=ShortURL&file=Add&mode=API"
        result = core().getShortenedLink(service_link, url, values)
        #<h3>URL has wrong format! Forgot 'http://'?</h3>
        if re.match('<h3>', result):
            result = url
        return result

class bit_ly:
    #curl "http://api.bit.ly/shorten?version=2.0.1&longUrl=http://www.google.com&format=json&login=bitlyapidemo&apiKey=R_0da49e0a9118ff35f52f629d2d71bf07"
    #Needs http://*
    #GET
    #TODO: Check for custom alias title
    def parseJSON(self, json_string):
        array = json.loads(json_string)
        #print(array)
        try:
            shorturl = array['results'][self.url]['shortUrl']
        except KeyError:
            #no shortUrl: errorCode > 0, no http://* or new API?
            return self.url

        if array['statusCode'] == 'OK' and array['errorCode'] == 0:
            result = shorturl
        else:
            result = self.url

        return result

    def get(self, args, url):
        self.url = url
        api_version = '2.0.1'
        username = args['username']
        apikey = args['api-key']
        if not username or not apikey:
            if not args['quiet']:
                print("ERROR: bit.ly requires a username and an API key")
            return url

        service_link = "http://api.bit.ly/shorten?version=%s&login=%s&apiKey=%s&longUrl=%s&format=json"
        url_response = core().getShortenedLink(service_link % (api_version, username, apikey, url), url)
        if not url_response == url:
            # If there was no error
            """ Response example:
            {
                "errorCode": 0, 
                "errorMessage": "", 
                "results": {
                    "http://www.google.com": { "hash": "2V6CFi", "shortKeywordUrl": "", "shortUrl": "http://bit.ly/8ttn", "userHash": "8ttn" }
                }, 
                "statusCode": "OK"
            }
            """
            result = self.parseJSON(url_response)
        else:
            result = url
        return result

class burnurl_com:
    #http://burnurl.com/?url=google.com&output=plain
    #TODO: Check for custom alias title
    def get(self, args, url):
        values = {
            'url'   : url,
            'output': 'plain',
        }
        service_link = "http://burnurl.com/"
        result = core().getShortenedLink(service_link, url, values)
        return result

class cli_gs:
    #curl "http://cli.gs/api/v1/cligs/create?url=http://www.example.com&appid=http://launchpad.net/surl"
    #GET
    #TODO: Check for custom alias title
    def get(self, args, url):
        appid = args['api-key']
        if not appid:
            appid = 'http://launchpad.net/surl'

        service_link = "http://cli.gs/api/v1/cligs/create?appid=%s&url=%s"
        result = core().getShortenedLink(service_link % (appid, url), url)
        return result

class sn_im:
    #http://snipr.com/site/getsnip?sniplink=http://www.example.com&snipformat=simple&snipuser=someuser&snipapi=someapi
    #TODO: Check for custom alias title
    def __init__(self, service_domain):
        self.service_link = 'http://' + service_domain + '/site/getsnip'

    def get(self, args, url):
        values = {
            'sniplink'  : url,
            'snipformat': 'simple',
            'snipapi'   : args['api-key'],
            'snipuser'  : args['username'],
        }
        result = core().getShortenedLink(self.service_link, url, values)
        return result

class decenturl_com:
    #curl "http://decenturl.com/api-get?u=youtube.com%2Fwatch%3Fv%3DpQHX-SjgQvQ&t=medieval"
    #GET
    #http://decenturl.com/tools
    #TODO: Check for custom alias title
    def parseJSON(self, json_string):
        array = json.loads(json_string)
        #print(array)
        if array[0] == "ok":
            result = array[3]
        else:
            result = self.url
        return result

    def get(self, args, url):
        self.url = url
        service_link = "http://decenturl.com/api-get?u=%s"
        url_response = core().getShortenedLink(service_link % (url), url)
        if not url_response == url:
            # If there was no error
            # Response example: ["ok", "example/example-web-page-2", "http://example.com/", "http://example.decenturl.com/example-web-page-2"]
            result = self.parseJSON(url_response)
        else:
            result = url
        return result

class digg_com:
    #curl "http://services.digg.com/url/short/create?url=http%3A%2F%2Fwww.google.com&appkey=http://launchpad.net/surl"
    #GET
    #http://apidoc.digg.com/ShortURLs
    #TODO: Check for custom alias title
    def parseXML(self, xml_string):
        doc = xml.dom.minidom.parseString(xml_string)
        first_shorturl = doc.getElementsByTagName("shorturl")[0]
        return first_shorturl.getAttribute("short_url")

    def get(self, args, url):
        appkey = args['api-key']
        if not appkey:
            appkey = 'http://launchpad.net/surl'

        service_link = "http://services.digg.com/url/short/create?appkey=%s&url=%s"
        url_response = core().getShortenedLink(service_link % (appkey, url), url)
        if not url_response == url:
            #i.e. If there was no error
            result = self.parseXML(url_response)
        else:
            result = url
        return result

class is_gd:
    #http://is.gd/api.php?longurl=http://www.example.com
    #GET
    #http://is.gd/api_info.php
    def get(self, args, url):
        service_link = "http://is.gd/api.php?longurl=%s"
        # hack over it, don't use POST
        result = core().getShortenedLink(service_link % url, url)
        return result

class kl_am:
    #curl "http://kl.am/api/shorten/?format=json&url=http%3A%2F%2Fgoogle.com"
    #GET
    #http://kl.am/api
    #FIXME: Use json
    def get(self, args, url):
        service_link = "http://kl.am/api/shorten/?format=text&url=%s"
        result = core().getShortenedLink(service_link % url, url)
        return result

class liip_to:
    #curl "http://liip.to/api/txt/?url=http://google.com&code="
    #GET
    def get(self, args, url):
        service_link = "http://liip.to/api/txt/?url=%s&code=%s"
        custom = args['title']
        result = core().getShortenedLink(service_link % (url, custom), url)
        return result

class metamark_net:
    #curl "http://metamark.net/api/rest/simple?long_url=http://www.google.com"
    #GET
    def get(self, args, url):
        service_link = "http://metamark.net/api/rest/simple?long_url=%s"
        result = core().getShortenedLink(service_link % (url), url)
        return result

class tr_im:
    #curl -S "http://api.tr.im/api/trim_url.json?url=http://www.google.com/youhavegottobetestingmetoo&custom=blarblarblar2"
    #GET or POST
    #http://tr.im/website/api#trim_url

    #url  	Required  	The destination URL to be shortened.
    #custom 	Optional 	A custom URL that is preferred to an auto-generated URL.
    #searchtags 	Optional 	A search string value to attach to a tr.im URL.
    #privacycode 	Optional 	A string value that must be appended after the URL.
    #newtrim 	Optional 	If present with any value, it will force the creation of a new tr.im URL.
    #api_key 	Optional 	An application API key assigned to your application.
    #sandbox 	Optional 	If present with any value a test data set will be returned, and no URL created. This is intended for testing so that you do not consume API calls or insert pointless data while in development.
    #username 	Optional 	A tr.im username that you would like to attach the URL to.
    #password 	Optional 	The password for the tr.im username referenced above.
    
    def parseJSON(self, json_string):
        array = json.loads(json_string)
        if array['status']['result'] == "OK":
            result = array['url']
        else:
            result = self.url
        return result

    def get(self, args, url):
        self.url = url
        values = {
            'url': url,
            'custom': args['title'],
            'api_key': args['api-key'],
            'username': args['username'],
            'password': args['password'],
        }
        service_link = "http://api.tr.im/api/trim_url.json"
        url_response = core().getShortenedLink(service_link, url, values)
        if not url_response == url:
            # If there was no error
            # Response example:
            # {"trimpath":"blarblarblar2", 
            # "reference":"3FxBAvOXb28YK6EZ6f6PpfYdiN851R", 
            # "trimmed":"19\/06\/2009", 
            # "destination":"http:\/\/www.google.com\/youhavegottobetestingmetoo", 
            # "trim_path":"blarblarblar2", "domain":"google.com", 
            # "url":"http:\/\/tr.im\/blarblarblar2","visits":0, 
            # "status":{"result":"OK","code":"200","message":"tr.im URL Added."}, 
            # "date_time":"2009/06/19 00:44:40 -0400"}
            result = self.parseJSON(url_response)
        else:
            result = url
        return result

class turl_ca:
    #curl -S "http://turl.ca/api.php?url=http://www.google.com&tag=moofoocoo"
    #GET
    # TODO: Use json:
    # curl "http://turl.ca/json.php?url=http://www.google.com&tag=moofoocoo"
    # ({"status":"ok","tag":"moofoocoo","turl":"http:\/\/turl.ca\/moofoocoo"})
    # curl "http://turl.ca/json.php?url=http://www.google.ca&tag=moofoocoo"
    # ({"status":"fail","error":"Tag already exists."})

    def errorcheck(self, tstring):
        reply = tstring.split(":")
        if reply[0] == "SUCCESS":
            result = reply[1]
        elif reply[0] == "ERROR":
            result = self.url
        return result

    def get(self, args, url):
        self.url = url
        tag = args['title']
        service_link = "http://turl.ca/api.php?url=%s&tag=%s"
        result = core().getShortenedLink(service_link % (url, tag), url)
        result2 = self.errorcheck(tstring)
        return result

class ur_ly:
    #curl "http://ur.ly/new.json?href=http://www.google.com"
    #Needs http://*
    #GET
    def parseJSON(self, json_string):
        array = json.loads(json_string)
        #print(array)
        codeurl = array['code']
        result = self.service_root + codeurl
        return result

    def get(self, args, url):
        self.service_root = 'http://ur.ly/'
        service_link = "http://ur.ly/new.json?href=%s"
        url_response = core().getShortenedLink(service_link % (url), url)
        if not url_response == url:
            # If there was no error
            # Response example: {"code":"e","href":"http://www.google.com"}
            result = self.parseJSON(url_response)
        else:
            result = url
        return result

class zz_gd:
    #curl "http://zz.gd/api-create.php?url=http://www.ingoal.info/archives/2008/07/08/twitter-updater/"
    #GET
    #Notes: Soon xml/json
    def get(self, args, url):
        service_link = "http://zz.gd/api-create.php?url=%s"
        result = core().getShortenedLink(service_link % (url), url)
        if result == "spam":
            result = url
        return result

